/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation;

import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.ComboBoxViewerCellEditor;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.wb.swt.SWTResourceManager;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.model.INamedCommentedElement;
import org.fortiss.tooling.kernel.service.IPersistencyService;

/**
 * 
 * @author eder
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public class GenericAnnotationView extends AnnotationViewPartBase {

	/** mapping of {@link IAnnotatedSpecification} -> TableViewerColumn */
	private HashMap<Class<? extends IAnnotatedSpecification>, TableViewerColumn> columns =
			new HashMap<Class<? extends IAnnotatedSpecification>, TableViewerColumn>();

	/** The table viewer */
	private TableViewer tableViewer;

	/** {@inheritDoc} */
	@Override
	public void createPartControl(Composite parent) {
		super.createPartControl(parent);
		Composite c = new Composite(parent, SWT.NULL);
		c.setLayout(new FillLayout(SWT.HORIZONTAL));

		tableViewer = new TableViewer(c, SWT.BORDER | SWT.FULL_SELECTION);
		Table table = tableViewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		final TableViewerColumn firstColumn = new TableViewerColumn(tableViewer, SWT.NONE);
		firstColumn.getColumn().setText("IModelElement");
		firstColumn.getColumn().setWidth(100);

		tableViewer.setContentProvider(new ArrayContentProvider());
		firstColumn.setLabelProvider(new ColumnLabelProvider() {
			/** {@inheritDoc} */
			@Override
			public String getText(Object element) {
				AnnotationEntry entry = (AnnotationEntry)element;
				String modelElementName = entry.getModelElement().getClass().getSimpleName();
				modelElementName = modelElementName.replace("Impl", "");
				firstColumn.getColumn().setText(modelElementName);
				return ((INamedCommentedElement)entry.getModelElement()).getName();// toString();
			}

			/** {@inheritDoc} */
			@Override
			public Color getBackground(Object element) {
				if(element instanceof AnnotationEntry) {
					AnnotationEntry data = (AnnotationEntry)element;
					if(data.getModelElement().equals(getCurrentlySelectedObject())) {
						return SWTResourceManager.getColor(0, 255, 0);
					}
				}
				return super.getBackground(element);
			}
		});

	}

	/** {@inheritDoc} */
	@Override
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		super.selectionChanged(part, selection);

		for(Entry<Class<? extends IAnnotatedSpecification>, TableViewerColumn> e : columns
				.entrySet()) {
			e.getValue().getColumn().dispose();
		}
		columns.clear();

		for(AnnotationEntry entry : values) {
			for(IAnnotatedSpecification spec : entry.getSpecificationsList()) {
				if(!isExistingColumn(spec)) {
					TableViewerColumn column = new TableViewerColumn(tableViewer, SWT.NONE);
					column.getColumn().setText(entry.getAnnotationName(spec.getClass()));
					column.getColumn().setWidth(100);
					columns.put(spec.getClass(), column);
					if(entry.getFixedSpecificationValues(spec.getClass()) == null) {
						column.setLabelProvider(new AnnotationLabelProvider(spec.getClass()));
						EditingSupport editingSupport =
								new StandardEditingSupport(column.getViewer(), spec.getClass());
						column.setEditingSupport(editingSupport);
					} else {
						column.setLabelProvider(new AnnotationLabelProvider(spec.getClass()));
						EditingSupport editingSupport =
								new ComboEditingSupport(column.getViewer(),
										entry.getFixedSpecificationValues(spec.getClass()),
										spec.getClass());
						column.setEditingSupport(editingSupport);
					}
				}

			}
		}

		if(!tableViewer.getTable().isDisposed())
			tableViewer.setInput(values);

	}

	/** Checks if column for given specification already exists */
	private boolean isExistingColumn(IAnnotatedSpecification specification) {
		if(columns.containsKey(specification.getClass()))
			return true;
		return false;
	}

	/** Editing support for combo table cells */
	private class ComboEditingSupport extends EditingSupport {

		/** Combo box cell editor */
		private ComboBoxViewerCellEditor cellEditor = null;

		/** Specification class of this column */
		private Class<? extends IAnnotatedSpecification> specClass;

		/**
		 * Constructor.
		 * 
		 * @param viewer
		 *            the column viewer
		 * @param values
		 *            the values for the combo box
		 * @param class1
		 */
		private ComboEditingSupport(ColumnViewer viewer, List<String> values,
				Class<? extends IAnnotatedSpecification> class1) {
			super(viewer);
			cellEditor =
					new ComboBoxViewerCellEditor((Composite)getViewer().getControl(), SWT.READ_ONLY);
			cellEditor.setLabelProvider(new LabelProvider());
			cellEditor.setContentProvider(new ArrayContentProvider());
			cellEditor.setInput(values);
			specClass = class1;

		}

		/** {@inheritDoc} */
		@Override
		protected CellEditor getCellEditor(Object element) {
			return cellEditor;
		}

		/** {@inheritDoc} */
		@Override
		protected boolean canEdit(Object element) {
			if(element instanceof AnnotationEntry) {
				AnnotationEntry data = (AnnotationEntry)element;
				return data.getSpecificationValue(specClass) != AnnotationEntry.NOVAL;
			}
			return true;
		}

		/** {@inheritDoc} */
		@Override
		protected Object getValue(Object element) {
			if(element instanceof AnnotationEntry) {
				AnnotationEntry data = (AnnotationEntry)element;
				return data.getSpecificationValue(specClass);
			}
			return null;
		}

		/** {@inheritDoc} */
		@Override
		protected void setValue(Object element, final Object value) {
			if(element instanceof AnnotationEntry && value instanceof String) {
				final AnnotationEntry data = (AnnotationEntry)element;

				if(!value.equals(data.getSpecificationValue(specClass))) {
					ITopLevelElement modelContext =
							IPersistencyService.INSTANCE.getTopLevelElementFor(data
									.getModelElement());
					modelContext.runAsCommand(new Runnable() {

						@Override
						public void run() {

							try {
								data.setSpecificationValue((String)value, specClass);
							} catch(IllegalArgumentException e) {
								// should not happen with combo boxes and given values
								MessageDialog
										.openError(
												new Shell(),
												"ERROR",
												"The value you entered does not have the desired type.\nDetailed message:\n" +
														e.getMessage());
							} catch(Exception e) {
								// e.printStackTrace();
								MessageDialog.openError(new Shell(), "ERROR", e.getMessage());
							}
						}
					});
					this.getViewer().refresh(true);
				}

			}
		}

	}

	/** Editing Suppourt for standard table cells */
	private class StandardEditingSupport extends EditingSupport {

		/** Text cell editor */
		private TextCellEditor cellEditor = null;
		/** Specification class of this column */
		private Class<? extends IAnnotatedSpecification> specClass;

		/** Constructor. */
		public StandardEditingSupport(ColumnViewer viewer,
				Class<? extends IAnnotatedSpecification> class1) {
			super(viewer);
			cellEditor = new TextCellEditor((Composite)getViewer().getControl(), SWT.NONE);
			specClass = class1;
		}

		/** {@inheritDoc} */
		@Override
		protected CellEditor getCellEditor(Object element) {
			return cellEditor;
		}

		/** {@inheritDoc} */
		@Override
		protected boolean canEdit(Object element) {
			if(element instanceof AnnotationEntry) {
				AnnotationEntry data = (AnnotationEntry)element;
				return data.getSpecificationValue(specClass) != AnnotationEntry.NOVAL;
			}

			return true;
		}

		/** {@inheritDoc} */
		@Override
		protected Object getValue(Object element) {
			if(element instanceof AnnotationEntry) {
				AnnotationEntry data = (AnnotationEntry)element;
				return data.getSpecificationValue(specClass);
			}
			return null;
		}

		/** {@inheritDoc} */
		@Override
		protected void setValue(Object element, final Object value) {
			if(element instanceof AnnotationEntry && value instanceof String) {
				final AnnotationEntry data = (AnnotationEntry)element;

				if(!value.equals(data.getSpecificationValue(specClass))) {
					ITopLevelElement modelContext =
							IPersistencyService.INSTANCE.getTopLevelElementFor(data
									.getModelElement());
					modelContext.runAsCommand(new Runnable() {

						@Override
						public void run() {
							try {
								data.setSpecificationValue((String)value, specClass);
							} catch(IllegalArgumentException e) {
								// e.printStackTrace();
								MessageDialog
										.openError(
												new Shell(),
												"ERROR",
												"The value you entered does not have the desired type.\nDetailed message:\n" +
														e.getMessage());
							} catch(Exception e) {
								MessageDialog.openError(new Shell(), "ERROR", e.getMessage());
							}
						}
					});
					this.getViewer().refresh(true);
				}

			}
		}

	}

	/** Label provider for {@link AnnotationEntry}s */
	public class AnnotationLabelProvider extends ColumnLabelProvider {
		/** Specification class of this column */
		Class<? extends IAnnotatedSpecification> specClass;

		/**
		 * @param class1
		 */
		public AnnotationLabelProvider(Class<? extends IAnnotatedSpecification> class1) {
			specClass = class1;
		}

		/** {@inheritDoc} */
		@Override
		public String getText(Object element) {
			if(element instanceof AnnotationEntry) {
				AnnotationEntry data = (AnnotationEntry)element;
				return data.getSpecificationValue(specClass);
			}
			return "-";
		}

		/** {@inheritDoc} */
		@Override
		public Color getBackground(Object element) {
			if(element instanceof AnnotationEntry) {
				AnnotationEntry data = (AnnotationEntry)element;
				if(data.getModelElement().equals(getCurrentlySelectedObject())) {
					return SWTResourceManager.getColor(0, 255, 0);
				}
			}
			return super.getBackground(element);
		}

	}

	/** {@inheritDoc} */
	@Override
	public void setFocus() {
		// nothing to do

	}

	/** {@inheritDoc} */
	@Override
	public void dispose() {
		tableViewer.getTable().dispose();
		super.dispose();
	}
}
